
;;
;; Verifying the rb-tree package
;;

(require-extension srfi-13)
(require-extension testeez)
(require-extension rb-tree)

(define-macro (++! x) `(set! ,x (fx+ 1 ,x)))
(define-macro (++ x) `(fx+ 1 ,x))
(define-macro (--! x) `(set! ,x (fx- ,x 1)))
(define-macro (-- x) `(fx- ,x 1))


(define (rb-tree-test)
  (testeez "--> Inserting a set of numbers in a red-black tree"
	   
	   (test-define "" min-key 1) 
	   (test-define "" max-key 100)
	   
	   (test-define "" rb-tree (make-rb-tree (lambda (x y) (- x y))))
	   
	   (test-define "a hard-wired association between a key and a value"
			compute-assoc (lambda (key) (cons key (++ key))))
	   
	   (test/equal "" (rb-tree 'empty?) #t)
	   (test/equal "" (zero? (rb-tree 'size))  #t)
	   
	   (test-eval (string-concatenate (list "loading a sequence [" 
						(number->string min-key) ", " 
						(number->string max-key) "] in ascending order"))
		      (do ((i min-key (++ i))) ((> i max-key))
			(testeez (test/equal "" ((rb-tree 'put!) i (cdr (compute-assoc i))) #f)
				 (test/equal "" ((rb-tree 'get) i) (compute-assoc i)))))

  	   (test/equal "" (rb-tree 'size) (++ (- max-key min-key)))
  	   (test/equal "" (rb-tree 'empty?)  #f)
	   
  	   (test/equal "" ((rb-tree 'get) (++ min-key))  (compute-assoc (++ min-key)))
  	   (test/equal "" ((rb-tree 'get) (++ min-key) #f)  (compute-assoc (++ min-key)))
	   
  	   (test/equal "check looking up of non-existing keys" (not ((rb-tree 'get) (-- min-key) #f)) #t)
  	   (test/equal "" ((rb-tree 'get) (++ max-key) (lambda () 1)) 1)
	   
  	   (test-eval "clear the rb-tree" (rb-tree 'clear!))
	   
  	   (test/equal "" (rb-tree 'empty?) #t)
  	   (test/equal "" (zero? (rb-tree 'size))  #t)
	   
  	   (test-eval "reloading the same seq in descending order and then deleting" 
  		      (do ((i max-key (-- i))) ((< i min-key))
  			(testeez (test/equal "" ((rb-tree 'put!) i (cdr (compute-assoc i))) #f)
				 (test/equal "" ((rb-tree 'get) i) (compute-assoc i))
				 (test/equal "" (if ((rb-tree 'delete!) i) #t #f) #t))))
				 

  	   (test/equal "" (zero? (rb-tree 'size)) #t)
	   
  	   (test-eval "loading the rb-tree again in a \"random\" order" 
  		      (do ((i min-key) (j max-key) (direction #t (not direction)))
  			  ((< j i))
  			(cond
  			 (direction
  			  (testeez (test/equal "" ((rb-tree 'put!) i (cdr (compute-assoc i))) #f))
  			  (++! i))
  			 (else
  			  (testeez (test/equal "" ((rb-tree 'put!) j (cdr (compute-assoc j))) #f))
  			  (--! j)))))
	   
  	   (test-eval "looking up the elements in  the rb-tree" 
		      (do ((i min-key (++ i))) ((> i max-key))
			(testeez (test/equal "" ((rb-tree 'get) i) (compute-assoc i)))))

	   (test/equal "using fold to sum the elements in the rb-tree" 
		       ((rb-tree 'fold) (lambda (x sum) (+ x sum)) 0) 
		       (* 50 (+ (+ 1 min-key) (+ 1 max-key))))

	   (test-define "Using 'map to create a copy of tree with each element x mapped to x*10" 
			rb-tree-x10 ((rb-tree 'map) (lambda (x) (* x 10))))

	   (test-define "a hard-wired association between a key and a value multiplied by 10"
			compute-assoc-x10 (lambda (key) (cons key (* 10 (++ key)))))

  	   (test-eval "looking up the elements in the x10 rb-tree" 
		      (do ((i min-key (++ i))) ((> i max-key))
			(testeez (test/equal "" ((rb-tree-x10 'get) i) (compute-assoc-x10 i)))))))





(rb-tree-test)

